/****
 *
 * Implementao do mdulo Leitura
 *
 * Autor:
 *
 * Data de Criao:
 * ltima modificao:
 *
 * Descrio Geral: Prov funes para leitura e validao de dados etc.
 *
 ****/

#include <stddef.h>
#include <errno.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "Leitura.h"

#define TAMANHO_DO_ARRANJO  50  /* Tamanho do arranjo que ir  	  */
								/* armazenar o contedo do buffer */

 /***************************************************************************/
 /*  Funes LOCAIS a este arquivo. Estas funes servem apenas como		    */
 /*  auxlio na implementao do mdulo. Portanto, elas no so exportadas. */
 /***************************************************************************/

/****
 *
 * A funo ImprimeMensagemDeErro() apresenta mensagens de erro no stream padro
 * de erros.
 *
 * Argumentos (entrada): formato - string de formao do mesmo tipo usado por printf()
 *						 ... - representa aquilo que ser impresso usando formato
 *
 * Retorno: aquilo que  retornado pela funo vfprintf() (consulte o Help).
 *
 ****/
static unsigned ImprimeMensagemDeErro(const char *formato, ...)
{
   va_list argumentos;
   int     resultado;

   va_start (argumentos, formato); /* Faz argumentos apontar para o primeiro     */
                                   /* argumento da lista de argumentos variveis */

   resultado = vfprintf(stderr, formato, argumentos);

   va_end (argumentos);  /* Encerra o processamento da lista de argumentos variveis */

   return resultado;
}

/****
 *
 * A funo IndicaErro() indica a posio aonde um erro ocorre
 * na entrada de dados do usurio.
 *
 * Argumentos (entrada): str - string que representa a entrada de dados
 *						 ptrErro - ponteiro para o primeiro caracter que
 *											  causa o erro.
 * Retorno: Nada
 *
 ****/

static void IndicaErro(const char *str, const char *ptrErro)
{
   static const char *const mensagem = "Entrada ilegal. Foi encontrado "
                                       "um erro na posicao indicada abaixo:\n";

   if (errno != 0) {  /* Foi detectado um erro na converso */
      ImprimeMensagemDeErro("%s\n", strerror(errno)); /* Imprime mensagem do sistem */
      ImprimeMensagemDeErro(mensagem);  /* Imprime parte inicial da indicao de erro */
   } else
      ImprimeMensagemDeErro(mensagem);  /* Idem */

   ImprimeMensagemDeErro("    %s\n", str);  /* Imprime o string lido na entrada */
   ImprimeMensagemDeErro("    %*c\n", (int) (ptrErro - str) + 1, '^');
}

 /***************************************************************************/
 /*                                                                         */
 /* 						   *** Funes Globais ***                      */
 /*                                                                         */
 /***************************************************************************/

/****
 *
 * A funo LeValor() l um string no meio de entrada padro, converte-o
 * para o tipo double e verifica se o valor convertido satisfaz os
 * critrios especificados
 *
 * Argumentos: Sada: valor - ponteiro para double que apontar para o
 *                            valor lido em caso de sucesso. Caso a funo
 *                            no seja bem sucedida, *valor  indefinido
 *             Entrada: pfVerifica - ponteiro para uma funo de verificao,
 *                                   que recebe um valor double como entrada e
 *                                   retorna 1 se o valor lido satisfaz os
 *                                   critrios de validade e 0, em caso contrrio
 *
 * Retorno: 1, se bem sucedida ou 0, caso contrrio.
 *
 ****/
unsigned LeValor(double *valor, unsigned (*pfVerifica)(double))
{
   size_t  nCaracteresLidos; /* O nmero de caracteres lido na entrada padro */
   char    strEntrada[TAMANHO_DO_ARRANJO]; /* Arranjo que conter o string lido */
   char   *ptrFinal; /* Aponta para o caractere que causar o final da converso */

   while (1) {
      printf("Introduza um numero de ponto flutuante: ");
      fflush(stdout); /* Garante que o usurio l o prompt imediatamente */

    					/* L a entrada e armazena-a no arranjo strEntrada */
      if (!fgets(strEntrada, sizeof(strEntrada), stdin))
         return 0;  /* O stream de entrada est vazio. */
                    /* No h nada mais a ser feito.   */

      nCaracteresLidos = strlen(strEntrada); /* O comprimento do string no     */
                                             /* arranjo strEntrada corresponde */
                                             /* ao nmero de caracteres lidos  */

      if (strEntrada[nCaracteresLidos-1] == '\n') { /* Testa se '\n' foi lido */
         strEntrada[--nCaracteresLidos] = '\0'; /* Remove o caracter '\n' e subtrai */
                                                /* 1 do nmero de caracteres lidos  */

         errno = 0;
         *valor = strtod(strEntrada, &ptrFinal);  /* Tenta converter o string */

         if (!errno && nCaracteresLidos && !*ptrFinal) /* Nenhum erro detectado */
            break;  /* Sai do while */

         IndicaErro(strEntrada, ptrFinal);
      } else {  		/* O usurio digitou caracters demais.				*/
         LimpaBuffer(stdin); /* Limpa o buffer de entrada que contm pelo menos '\n' */
         ImprimeMensagemDeErro("A entrada foi muito grande. O numero maximo "
         				   "de caracteres e' %d.\n", TAMANHO_DO_ARRANJO);
      } /* if */
   } /* while */

  		/* Determina se o valor satisfaz outros critrios especficos  */

   if (!pfVerifica)
      return 1;

   if (pfVerifica(*valor))  /* Utiliza o ponteiro para funo passado como */
      return 1;			    /* parmetro para chamar a funo de validao */
   else {
      ImprimeMensagemDeErro("O valor %lf nao satisfaz os criterios do programa.\n", *valor);
      return 0;
   }
}

/****
 *
 * A funo LeString() l um string no meio de entrada contendo um nmero
 * mximo de caracteres especificado
 *
 * Argumentos: Sada: buffer - arranjo de caracteres que conter o string lido
 *             Entrada: nElementos - tamanho do arranjo buffer
 *
 * Retorno: ponteiro para o string lido ou NULL, caso contrrio.
 *
 ****/
char *LeString(char * buffer, unsigned nElementos)
{
   unsigned contador = 0;   /* Conta o nmero de caracteres lidos */
   unsigned caracterLido;

   		/* L no mximo nElementos - 1 caracteres ou at encontrar '\n' */
   caracterLido = getchar();
   while ((caracterLido != '\n') && (contador < nElementos-1)) {
      buffer[contador++] = (char) caracterLido;
      caracterLido = getchar();
   }

   buffer[contador] = '\0'; /* Termina o string */

   if ((caracterLido == '\n') || (getchar() == '\n'))  /* Tudo bem */
      return buffer;
   else {      /* Usurio digitou caracteres demais */
      LimpaBuffer(stdin); /* Limpa a sujeira deixada pelo usurio */
      return (char *) 0;
   }
}

/****
 *
 * A funo LeEValidaString() l um string no meio de entrada contendo um nmero
 * mximo de caracteres especificado e que satisfaz um critrio especificado
 *
 * Argumentos: Sada: buffer - arranjo de caracteres que conter o string lido
 *             Entrada: nElementos - tamanho do arranjo buffer
 *                      prompt - a indio de entrada apresentada ao usurio
 *                      erro - a mensagem de erro apresentada ao usurio
 *                      valida - ponteiro para uma funo de validao do
 *                               string; se este argumento for NULL, no existe
 *                               critrio de validao.
 *
 * Retorno: ponteiro para o string lido e validado ou NULL, caso contrrio.
 *
 ****/
char *LeEValidaString(char * buffer, unsigned nElementos, const char* prompt,
									   const char* erro, tFuncValidaString valida)
{
   char *stringLido;

   printf("%s", prompt); /* Jamais faa isto: printf(prompt); pois, se prompt */
   							 /* contiver o caracter '%', voc estar perdido      */

   if (!(stringLido = LeString(buffer, nElementos))) {
      printf("%s", erro); /* Jamais faa isto: printf(erro); */;
      return stringLido; /* Neste caso, stringLido  NULL */
   } else if (valida && !valida(stringLido)) { /* A funo de validao existe */
      printf("%s", erro); 						  /* e o critrio no foi satisfeito */
      return (char *) 0;
   }

   return stringLido;
}

/****
 *
 * Funo: LimpaBuffer
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 10/06/2002
 * ltima modificao: 10/06/2002
 *
 * Descrio: L e descarta caracteres at o final da linha corrente
 *					do arquivo texto que recebe como argumento
 *
 * Entrada: Ponteiro para a estrurtur FILE de um arquivo texto
 * Sada: Nada
 * Retorno: Nada
 *
 ****/
void LimpaBuffer(FILE * arquivoPtr)
{
   int valorLido;

   while( (valorLido = fgetc(arquivoPtr)) != EOF && valorLido != '\n' )
      ; /* Corpo Vazio */
}



